///////////////////////////////////////////////////////////////////////////////////
//
// PLL Phase detector (frequency synthesis type)
//
//
//
//                  Author: Leonard Dieguez
//                  System IO specialist Field Applications Engineer
//
//                  Video Applications
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 10, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision: Ver. 1.0
//
//
//------------------------------------------------------------------------------
//Description of module:
//
// This module compares two frequecies, one from a reference and the other from a
// VCO or VCXO then generates a correction to be applied to the VCO /VCXO that is
// proportional to the error measured. A one Mega-Hertz comparison frequency 
// was used in this module for low phase noise. Lower comparison frequencies can 
// be used and the user may want to experiment with this. 
//
// This is provided for example purposes and has be verified in hardware running 
// at 270Mbps using 27MHz oscillator and the Maxim 270Mhz VCO.
//
// see See application note XAPP250 for more information the phase dectector.
/////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1 ps


module  pd_27to270 (
	//inputs
	vco,
	refclk,
	reset,
	vcoisfast,
	vcoisslow,
	vco_tc,
	refclk_tc
	);

///////////////////////////////////////////////////
// port declarations
input	vco;
input	refclk;
input	reset;

output	vcoisfast;
output	vcoisslow;
output	vco_tc ;
output	refclk_tc;

///////////////////////////////////////////////////
wire	vcoisfast;
wire	vcoisslow;
wire	pd_rst;



////////////////////////////////////////////////////////
//divide counter,  
//divide vco (270MHz) by 270 to get 1MHz the comparison 
//frequency.
//512 - 243(0xF3; 10'b00_1111_0011) + 1 = 270 .
////////////////////////////////////////////////////////
reg	[9:2]	vcocount;	//10bits - 2bits = 8bits
reg	[3:0]	vcocntlow;
wire		vco_tc       /* synthesis syn_keep = 1 */ ;
wire		vcocntlow_tc /* synthesis syn_keep = 1 */ ;

assign vco_tc = vcocount[9] | reset ;
assign vcocntlow_tc = vcocntlow[3] ;
////////////////////////////////////////////////////////

always @ (posedge vco ) begin 
	if (vco_tc) begin
		vcocount 	<= #1 8'b00_1111_00 ;
	end
	else if( vcocntlow_tc ) begin
		vcocount	<= #1 vcocount + 1;
	end
end

////////////////////////////////////////////////////////
//lower 2 bits of the vco counter.
//shift reg counter is one hot code;
//bit 0 is 2'b00
//bit 1 is 2'b01
//bit 2 is 2'b10
//bit 3 is 2'b11
always @ (posedge vco ) begin 
	if (vco_tc) begin
		vcocntlow	<= #1 4'b1000;	// init to 3
	end
	else begin
		vcocntlow	<= #1 {vcocntlow[2:0], vcocntlow[3]};
	end
end

//////////////////////////////////////////////////////////////
reg	[5:2]	refclkcount;
reg	[3:0]	refclkcntlow ;
wire		refclk_tc /* synthesis syn_keep = 1 */ ;
wire		refclkcntlow_tc /* synthesis syn_keep = 1 */ ;

assign refclk_tc = refclkcount[5] | reset;
assign refclkcntlow_tc = refclkcntlow[3];
/////////////////////////////////////////////////////////////
//ref clock divider
//divide the ref clock (27MHz) by 27 to get compare freq of 1MHz
//32 - 6(0x06; 6'b00_0110) + 1 = 27
always @ (posedge refclk ) begin 
	if (refclk_tc) begin 
		refclkcount 	<= #1 4'b00_01;
	end
	else if ( refclkcntlow_tc ) begin
		refclkcount	<= #1 refclkcount + 1;
	end
end

///////////////////////////////////////////
//lower 2 bits of the ref counter.
//shift reg counter is one hot code;
//bit 0 is 2'b00
//bit 1 is 2'b01
//bit 2 is 2'b10
//bit 3 is 2'b11
always @ (posedge refclk ) begin 
	if (refclk_tc) begin
		refclkcntlow	<= #1 4'b0100;  //the "10" part of the 0x06
	end
	else begin
		refclkcntlow	<= #1 {refclkcntlow[2:0], refclkcntlow[3]};
	end
end

/////////////////////////////////////////////////////////
//phase detect for reference
//////////////////////////////////////////////
// set on the rising edge of the vco /ref
//pdetfast is active when the vco is faster than the ref xtal
//pdetslow is active when the vco is slower than the ref xtal
FDC pdfastreg     ( .D(1'b1), .C(vco_tc),    .CLR(pd_rst), .Q(vcoisfast));
FDC pdslowreg     ( .D(1'b1), .C(refclk_tc), .CLR(pd_rst), .Q(vcoisslow));
LUT2 pdrst_and2_0 (.I0(vcoisfast), .I1(vcoisslow), .O(pd_rst));
// (OLD ISE) synthesis_attribute init of pdrst_and2_0 is 4'h8 ;
// synthesis attribute init of pdrst_and2_0 is 8 ;

//defparam pdrst_and2_0.INIT = 4'h8;  //pdetfast & pdetslow



endmodule
