///////////////////////////////////////////////////////////////////////////////////
//
// PLL Phase detector (CDR type)
//
//
//
//                  Author: Leonard Dieguez
//                  System IO specialist Field Applications Engineer
//
//                  Video Applications
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 10, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision: Ver. 2.2
//															
//
//------------------------------------------------------------------------------
//Description of module:
//
// This module compares two phases, one from data received and the other from a
// VCO or VCXO then generates a correction to be applied to the VCO /VCXO that is
// proportional to the error measured.  
//
// This is provided for example purposes and has be verified in hardware running 
// at 270Mbps using the Maxim 270Mhz VCO.
//
// see See application note XAPP250 for more information the phase dectector.
/////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1 ps

module  pd_clk2data_v22 (
	//inputs
	clk,
	reset,
	data,
	//outputs
	data_out,
	mkclkfaster,
	mkclkslower
	);

///////////////////////////////////////////////////
// port declarations
input	clk;
input	reset;
input	data;


output	[1:0] data_out;
output	mkclkfaster;
output	mkclkslower;


///////////////////////////////////////////////////
reg	[31:0]	pd_2R;
reg	[15:0]	phase;
reg	[15:0]	phaseR;
reg	[15:0]	phase2R;

reg		phase_en /* synthesis syn_direct_enable = 1 syn_keep=1*/;
reg		mkclkfaster ;
reg		mkclkslower ;


wire	[31:0]	data_taps;


////////////////////////////////////////////////////////////////
//This is the main delay chain used for phase detection
muxdelchain32 pddelay(
	.clk(clk),
	.reset(reset),
	.cin(data),
	.out(data_taps),
	.data(data_out)
);



reg	[3:0]	clkcntlow;

wire		clkcntlow_tc /* synthesis syn_direct_enable = 1 syn_keep=1*/;


///////////////////////////////////////////////////////////////////
//divide clk
///////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////
//lower 2 bits of the vco counter.
//combined reset is   (0x29) 
//so the low 2 bits reset to 3
//shift reg counter is one hot code;
//bit 0 is 2'b00
//bit 1 is 2'b01
//bit 2 is 2'b10
//bit 3 is 2'b11
always @ (posedge clk or posedge reset) begin 
	if (reset) begin
		clkcntlow	<= #1 4'b0001;
	end
	else begin
		clkcntlow	<= #1 {clkcntlow[2:0], clkcntlow[3]};
	end
end

assign clkcntlow_tc = clkcntlow[3];

////////////////////////////////////////////////////////////////////
reg	[1:0]	pd_enable /* synthesis syn_direct_enable = 1 syn_keep=1 */;
////////////////////////////////////////////////////////////////////
// pd_enable is used to enable clk(VCO) updates. 
// pd_enable can be used as a clock enable that is active every 
// 4 clocks VCO phase updates are governed by the rate of 
// pd_enable bit that is used.
always @ (posedge clk or posedge reset ) begin 
	if (reset) begin
		pd_enable	<= #1 12'b0 ;
	end
	else begin 
		//div by 4
		pd_enable	<= #1 clkcntlow_tc ;
	end
end


//////////////////////////////////////////////
//register the delayed data (taps)
//part of the phase detector for use with the VCO
//implimentation.
always @ (posedge clk or posedge reset ) begin 
	if (reset) begin
		pd_2R		<= #1 32'b0 ;
		phase		<= #1 16'b0 ;
		phase_en	<= #1 1'b0  ;
		phaseR		<= #1 16'b0 ;
		phase2R		<= #1 16'b0 ;
		end
		
	else  begin
		pd_2R		<= #1 data_taps;
		
		//detect a trasition during this period.
		phase_en	<= #1	pd_2R[31] & ~pd_2R[0] ;

		//detect the rising edge
		//as it travels up the delay chain
		phase[0]	<= #1 	pd_2R[2] & ~pd_2R[0] ;
		phase[1]	<= #1 	pd_2R[4] & ~pd_2R[2] ;	
		phase[2]	<= #1 	pd_2R[6] & ~pd_2R[4] ;
		phase[3]	<= #1 	pd_2R[8] & ~pd_2R[6] ;
		phase[4]	<= #1 	pd_2R[10] & ~pd_2R[8] ;
		phase[5]	<= #1 	pd_2R[12] & ~pd_2R[10] ;
		phase[6]	<= #1 	pd_2R[14] & ~pd_2R[12] ;
		phase[7]	<= #1 	pd_2R[16] & ~pd_2R[14] ;
		
		//center.
		phase[8]	<= #1 	pd_2R[17] & ~pd_2R[16] ;
		phase[9]	<= #1 	pd_2R[19] & ~pd_2R[17] ;
		phase[10]	<= #1 	pd_2R[21] & ~pd_2R[19] ;
		phase[11]	<= #1 	pd_2R[23] & ~pd_2R[21] ;
		phase[12]	<= #1 	pd_2R[25] & ~pd_2R[23] ;
		phase[13]	<= #1 	pd_2R[27] & ~pd_2R[25] ;
		phase[14]	<= #1 	pd_2R[29] & ~pd_2R[27] ;
		phase[15]	<= #1 	pd_2R[31] & ~pd_2R[29] ;
		
		//only update when a valid phase hit occurs
		if (phase_en) begin
			phaseR		<= #1 phase;
		end	
		//store phaseR for one clock
		if(pd_enable) begin
			phase2R		<= #1 phaseR;
		end
		
	end
end


////////////////////////////////////////////////
wire		clkleads_tc ;
wire		clklags_tc ;

reg		clklags ;
reg		clkleads ;

reg	[10:0]	clkleadscnt ;
reg	[10:0]	clklagscnt ;

//////////////////////////////////////////////
//phaseR 8 to 15 clk lags
//phaseR 0 to 7  clk leads
//
// section to correct phase errors but not frequency error
// pd_enable is used to "weight" the correction values.
// for example, the farther from the center the error is 
// the more correction is imposed.
////////////////////////////////////////////////
//clock leads or lags counters.
//this is the phase detection filter.
//it will act as an integrator for the phase differences
//in the ideal case when both the data and the oscillator are 
//frequency locked but not phase locked this counter should output
//corrections for the phase error
/////////////////////////////////////////////////
always @ (posedge clk or posedge reset ) begin 
	if (reset) begin
		clklags		<= #1 0 ;
		clkleads	<= #1 0 ;
		clkleadscnt 	<= #1 0 ;
		clklagscnt	<= #1 0 ;	
		end
		
	else if ( pd_enable ) begin	//enable the counter every 4 clocks	

		//lead lag counters
	 	if (clkleads_tc) //use the msb as the sync reset for both counters
			clkleadscnt	<= #1 0 ;

		else if (phase2R[0])
			clkleadscnt 	<= #1 clkleadscnt + 8 ;
		else if (phase2R[1])
			clkleadscnt 	<= #1 clkleadscnt + 7 ;
		else if (phase2R[2])
			clkleadscnt 	<= #1 clkleadscnt + 6 ;
		else if (phase2R[3])
			clkleadscnt 	<= #1 clkleadscnt + 6 ;
		else if (phase2R[4])
			clkleadscnt 	<= #1 clkleadscnt + 4 ;
		else if (phase2R[5])
			clkleadscnt 	<= #1 clkleadscnt + 3 ;
		else if (phase2R[6])
			clkleadscnt 	<= #1 clkleadscnt + 2 ;
		else if (phase2R[7])
			clkleadscnt 	<= #1 clkleadscnt + 1 ;
		
		////////////////////////////////////////////////////////////////////
		if (clklags_tc)	//use the msb as the sync reset for both counters
			clklagscnt	<= #1 0 ;
		//else if (clklags)
		//else if (clklags & leadlagcnt_en)
		else if (phase2R[8])
			clklagscnt 	<= #1 clklagscnt + 1 ;
		else if (phase2R[9])
			clklagscnt 	<= #1 clklagscnt + 2 ;
		else if (phase2R[10])
			clklagscnt 	<= #1 clklagscnt + 3 ;
		else if (phase2R[11])
			clklagscnt 	<= #1 clklagscnt + 4 ;
		else if (phase2R[12])
			clklagscnt 	<= #1 clklagscnt + 5 ;
		else if (phase2R[13])
			clklagscnt 	<= #1 clklagscnt + 6 ;
		else if (phase2R[14])
			clklagscnt 	<= #1 clklagscnt + 7 ;
		else if (phase2R[15])
			clklagscnt 	<= #1 clklagscnt + 8 ;
	end

end

assign clkleads_tc = clkleadscnt[7] ;	//assign the terminal counts
assign clklags_tc = clklagscnt[7] ;

always @ (posedge clk or posedge reset ) begin 
	if (reset) begin
		mkclkfaster	<= #1 0 ;
		mkclkslower	<= #1 0 ;
	end
		
	else  begin
		mkclkfaster	<= #1 (clklags_tc) & pd_enable ;
		mkclkslower	<= #1 (clkleads_tc) & pd_enable ;
	end
end


endmodule

