///////////////////////////////////////////////////////////////////////////////
// Top level wraper for XAPP250 implementation example using Maxim VCO for CDR
//
//
//                  Author: Leonard Dieguez
//                  System IO specialist Field Applications Engineer
//
//                  Video Applications
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Updated:   Oct. 10, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision: Ver. 1.3
//															
// Other modules instanced in this design:
//
//------------------------------------------------------------------------------
//Description of module:
//
// This module instances modules that through control of a VCO / VCXO will preform 
// frequency synthesis and clock and data recovery (CDR).
//
// This is provided for example purposes and has be verified in hardware running 
// at 270Mbps using SDI and ASI video streams. See application note XAPP250 for 
// more information on the style CDR using the Maxim 270Mhz VCO.
/////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1 ps

module top_xapp250 (
	vco270M_p,			// 270MHz VCO pos clk input
	vco270M_n,			// 270MHz VCO neg clk input
	
	vcxo_27M_in,			// 27MHz VCXO clock input used as ref osc only 
					// (not VCXO)

	// SDI Port	
	sdi_rxd_p,			// SDI serial input bit stream 
	sdi_rxd_n,			// SDI serial input bit stream
	
	sdi_txd_p,			// SDI serial output for verification
	sdi_txd_n, 			// SDI serial output for verification

	dip_switches,			// selects user options

	sdi_vco_up,			// 270Mhz VCO faster control to loop filter
	sdi_vco_down,			// 270Mhz VCO slower control to loop filter
	

	test_points			// selected 10 bits of selected error counter 
);

input		vco270M_p;
input		vco270M_n;

input		vcxo_27M_in ;

input		sdi_rxd_p;
input		sdi_rxd_n;

output		sdi_txd_p;
output		sdi_txd_n;

input	[7:0]	dip_switches;

output	[10:1]	test_points;
	

output		sdi_vco_up;
output		sdi_vco_down;


/////////////////////////////////////////////////////////////////////////
// Signal definitions

wire		vco_270M;		// 270Mhz VCO after IOB
wire		vco_270M_global;	// 270Mhz VCO after BUFG

wire		sdi_rx_din;		// rx serial data input before input FF

wire [10:1]	test_out;		// test points
wire [7:0]	dipsw;			// buffered DIP switch signals

wire		VCC;
wire		GND;

wire [1:0]	dataout;		// data from CDR

reg [4:0]	vco270divreg ;		//vco_270M
wire		vco27M ;		//270 MHz vco divided by 10 for word clock


assign VCC = 1'b1;
assign GND = 1'b0;


///////////////////////////////////////////////////////////
// Global clock buffers
//
BUFG bufgVCO270 (.O(vco_270M),.I(vco_270M_ibufds)); //global buffer bit clk
BUFG bufgVCO27 (.O(vco_27M), .I(vco270divreg[0]));  //global buffer word clk


///////////////////////////////////////////////////////////
// Receiver Section
//
// CDR
phasedet_v01 phasedet(
	// inputs
	.vco			(vco_270M),		
	//.refclk		(clk54M_ref), //use pd_54to270
	.refclk			(vcxo_27M),   // use pd_27to270
	
	.reset			(reset),
	.data			(sdi_rx_din), // input data to CDR module
	.pd_select		(dipsw[1]),   // manual control over CDR phase detector
	//outputs
	.dataout		(dataout),	// 2 bits same data output
	.mkvcofaster		(make_vco_faster), //correction to make VCO faster
	.mkvcoslower		(make_vco_slower), //correction to make VCO slower
	//debug outputs
	.vco_tc			(vco_tc),	// terminal count at comparison freq
	.refclk_tc		(clk_tc)	// terminal count at comparison freq
);




/////////////////////////////////////////////////////
// divide VCO input (bit clock) by 10 to 
// make the word clock. see vco_27M gbuf
//
// register the recoverd data to retransmit
reg		sdi_tx_dout ;

always @ (posedge vco_270M or posedge reset ) begin 
	if (reset) begin
		vco270divreg	<= #1 5'b0 ;
		sdi_tx_dout	<= #1 1'b0 ;
	end
	else begin
		vco270divreg	<= #1 {vco270divreg[3:0],~vco270divreg[4]} ;
		sdi_tx_dout	<= #1 dataout[1];
	end
end



/////////////////////////////////////////////////////////
// DIP switches
IBUF_LVCMOS25 SW0 (.I(dip_switches[0]), .O(dipsw[0]));
// synthesis attribute LOC of SW0 is R4;
IBUF_LVCMOS25 SW1 (.I(dip_switches[1]), .O(dipsw[1]));
// synthesis attribute LOC of SW1 is R5;
IBUF_LVCMOS25 SW2 (.I(dip_switches[2]), .O(dipsw[2]));
// synthesis attribute LOC of SW2 is AB3;
IBUF_LVCMOS25 SW3 (.I(dip_switches[3]), .O(dipsw[3]));
// synthesis attribute LOC of SW3 is AB4;
IBUF_LVCMOS25 SW4 (.I(dip_switches[4]), .O(dipsw[4]));
// synthesis attribute LOC of SW4 is AC1;
IBUF_LVCMOS25 SW5 (.I(dip_switches[5]), .O(dipsw[5]));
// synthesis attribute LOC of SW5 is AC2;
IBUF_LVCMOS25 SW6 (.I(dip_switches[6]), .O(dipsw[6]));
// synthesis attribute LOC of SW6 is AD1;
IBUF_LVCMOS25 SW7 (.I(dip_switches[7]), .O(dipsw[7]));
// synthesis attribute LOC of SW7 is AD2;

////////////////////////////////////////////////////////////
// Test outputs
OBUF_LVCMOS25 TP1 (.I(vco_27M), .O(test_points[1]));
// synthesis attribute LOC of TP1 is A25;
OBUF_LVCMOS25 TP2 (.I(vco_tc), .O(test_points[2]));
// synthesis attribute LOC of TP2 is B26;
OBUF_LVCMOS25 TP3 (.I(clk_tc), .O(test_points[3]));
// synthesis attribute LOC of TP3 is C25;
OBUF_LVCMOS25 TP4 (.I(dataout[1]), .O(test_points[4]));
// synthesis attribute LOC of TP4 is C26;
OBUF_LVCMOS25 TP5 (.I(1'b0), .O(test_points[5]));
// synthesis attribute LOC of TP5 is D26;
OBUF_LVCMOS25 TP6 (.I(1'b0), .O(test_points[6]));
// synthesis attribute LOC of TP6 is E23;
OBUF_LVCMOS25 TP7 (.I(1'b0), .O(test_points[7]));
// synthesis attribute LOC of TP7 is E24;
OBUF_LVCMOS25 TP8 (.I(1'b0), .O(test_points[8]));
// synthesis attribute LOC of TP8 is E25;
OBUF_LVCMOS25 TP9 (.I(1'b0), .O(test_points[9]));
// synthesis attribute LOC of TP9 is E26;
OBUF_LVCMOS25 TP10 (.I(1'b0), .O(test_points[10]));
// synthesis attribute LOC of TP10 is G21;


///////////////////////////////////////////////////////////////
// SDI 270MHz VCO
// 270MHz VCO clock input
IBUFGDS_LVDSEXT_25_DCI VCO270 
	(.O(vco_270M_ibufds),.I(vco270M_p),.IB(vco270M_n));
// synthesis attribute LOC of vco270M_p	is AE13 ;
// synthesis attribute LOC of vco270M_n	is AD13 ;



//270MHz VCO control outputs
OBUF_LVDCI_33 SDVCO_U (.I(make_vco_faster), .O(sdi_vco_up));
// synthesis attribute LOC of sdi_vco_up is "M3" ;
OBUF_LVDCI_33 SDVCO_D (.I(make_vco_slower), .O(sdi_vco_down));
// synthesis attribute LOC of sdi_vco_down is "M4" ;	

///////////////////////////////////////////////////////////////	
// SDI 27MHz VCXO
IBUF_LVDCI_25 VCXO27M (.I(vcxo_27M_in), .O(vcxo_27M));
// synthesis attribute LOC of vcxo_27M_in is AA15 ;

//OBUF_LVDCI_33 SDVCXO_U (.I(GND), .O(vcxo_27M_up));
// synthesis attribute LOC of vcxo_27M_up is M2 ;
//OBUF_LVDCI_33 SDVCXO_D (.I(GND), .O(vcxo_27M_down));
// synthesis attribute LOC of vcxo_27M_down is M1 ;

//////////////////////////////////////////////
// 54MHz input clock buffer
// used if 54MHz ref is desired
/*
IBUFGDS_LVDS_25_DCI CLK54 (
	.O		(clk54M_ref),
	.I		(clk54M_p),
	.IB		(clk54M_n));
*/
// synthesis attribute LOC of clk54M_p is D14;
// synthesis attribute LOC of clk54M_n is E14;

//////////////////////////////////////////////////////
// SDI port (J6)
// Input buffers for the serial data entering the FPGA
IBUFDS_LVDS_25_DCI MRSDDAT (
	.O		(sdi_rx_din),
	.I		(sdi_rxd_p),
	.IB		(sdi_rxd_n));
// synthesis attribute LOC of sdi_rxd_p is Y14;
// synthesis attribute LOC of sdi_rxd_n is AA14;

// SDI-Tx Port transmit data buffer
OBUFDS_LVDS_25 SDI_TXD (.I(sdi_tx_dout), .O(sdi_txd_p), .OB(sdi_txd_n));
// synthesis attribute LOC of sdi_txd_p is AC15;
// synthesis attribute LOC of sdi_txd_n is AD15;



///////////////////////////////////////////////////
// Output buffers for signals controlling the 27Mhz 
// VCXO 
//OBUF_LVDCI_33 VCXO27M_LPFEN (.O(vcxo_27M_sel), .I(VCC));
// synthesis attribute LOC of vcxo_27M_sel is N7;

endmodule
